#########################################################################################################
##
## Illustration
##
## Supplementary to the manuschipt:
##
##        Comparing modeling approaches for discrete event simulations of competing 
##        events based on censored individual patient data: a simulation study and 
##        illustration in colorectal cancer
##
##        by K. Degeling, M.J. IJzerman, C.G.M. Groothuis-Oudshoorn, M.D. Franken, 
##        M. Koopman,  and H. Koffijberg
##
##        DOI: 
## 
## This code illustrates how all functions defined in the provided R scripts can be used to implement
## individual patient time-to-event data in a discrete event simulation. The illustration includes both 
## examples on data analysis for the base-case analysis, as well as an example of performing a 
## probabilistic sensitivity analysis by bootstrapping the dataset to reflect parameter uncertainty in 
## the estimated time-to-event related proabilities and distribution parameters. The data used is an 
## extension of the exemplary data presented in the manuscript. This code also illustrates how the 
## results of the functions can be used to perform discrete event simulations.
##
#########################################################################################################

## Initialization ----

# Clear the workspace
rm(list=ls());

# Set the working directory to the folder which contains all R files
setwd("...SET DIRECTORY INCLUDING ALL PROVIDED R FILES...");

# Load all functions
source("function censor.R");
source("function haz.R");
source("function cs_dens.R");
source("function cs_cif.R");
source("function cs_loglik.R");
source("function sd_cif.R");
source("function sd_loglik.R");
source("function ESD_analysis.R");
source("function ESD_simulation.R");
source("function ESPD_analysis.R");
source("function ESPD_simulation.R");
source("function UDR_analysis.R");
source("function UDR_simulation.R");
source("function MDR_analysis.R");
source("function MDR_simulation.R");

# Install the required Packages
#install.packages("mixtools");       # Package for simulating from mixture distributions

# Load the required Packages
library("stringr"); 
library("nnet");
library("mixtools");       # Package for simulating from mixture distributions

# Load the exemplary data object 'IPD' 
load("data IPD.RData");

# Censor the exemplary data
IPD_censored <- censor(times=IPD$time, events=IPD$event, proportion=0.20, classification="Censored");




## Base-case analysis: ESD Modeling Approach ----

## Data analysis

# Call the 'ESD_analysis' function
ESD_in <- ESD_analysis(time=IPD_censored$Time, event=IPD_censored$Event, censored="Censored");

# Assess the data analysis results
ESD_in;

## Simulation

# Perform simulation
ESD_out <- ESD_simulation(parameters=ESD_in, n_sim=1000);

# View results
View(ESD_out);




## Base-case analysis: ESPD Modeling Approach ----

## Data analysis

# Call the 'ESPD_analysis' function
ESPD_in <- ESPD_analysis(time=IPD_censored$Time, event=IPD_censored$Event, censored="Censored");

# Assess the data analysis results
ESPD_in;

## Simulation

# Perform simulation
ESPD_out <- ESPD_simulation(parameters=ESPD_in, n_sim=1000);

# View results
View(ESPD_out);




## Base-case analysis: UDR Modeling Approach ----

## Data analysis

# Call the 'UDR_analysis' function
UDR_in <- UDR_analysis(time=IPD_censored$Time, event=IPD_censored$Event, censored="Censored");

# Assess the data analysis results
UDR_in;

## Simulation

# Perform simulation
UDR_out <- UDR_simulation(parameters=UDR_in, n_sim=1000);

# View results
View(UDR_out);




## Base-case analysis: MDR Modeling Approach ----

## Data analysis

# Call the 'MDR_analysis' function
MDR_in <- MDR_analysis(time=IPD_censored$Time, event=IPD_censored$Event, censored="Censored");

# Assess the data analysis results
MDR_in;

## Simulation

# Perform simulation
MDR_out <- MDR_simulation(parameters=MDR_in, n_sim=1000);

# View results
View(MDR_out);




## Visualize outcomes for the base-case analysis

# Density plot for event "Progression"
plot(density(IPD$time[IPD$event=="Progression"]), type='l', col="black", lty=1, lwd=2, main="Event 'Progression'", xlab="Time");
lines(density(ESD_out$Time[ESD_out$Event=="Progression"]), col="red", lty=1, lwd=2);
lines(density(ESPD_out$Time[ESPD_out$Event=="Progression"]), col="blue", lty=1, lwd=2);
lines(density(UDR_out$Time[UDR_out$Event=="Progression"]), col="green", lty=1, lwd=2);
lines(density(MDR_out$Time[MDR_out$Event=="Progression"]), col="purple", lty=1, lwd=2);
legend("right", c("Uncensored Data", "ESD Approach", "ESPD Approach", "UDR Approach", "MDR Approach"), 
       col=c("black", "red", "blue", "green", "purple"), lty=1, lwd=2);

# Density plot for event "Death"
plot(density(IPD$time[IPD$event=="Death"]), type='l', col="black", lty=1, lwd=2, main="Event 'Death'", xlab="Time");
lines(density(ESD_out$Time[ESD_out$Event=="Death"]), col="red", lty=1, lwd=2);
lines(density(ESPD_out$Time[ESPD_out$Event=="Death"]), col="blue", lty=1, lwd=2);
lines(density(UDR_out$Time[UDR_out$Event=="Death"]), col="green", lty=1, lwd=2);
lines(density(MDR_out$Time[MDR_out$Event=="Death"]), col="purple", lty=1, lwd=2);
legend("right", c("Uncensored Data", "ESD Approach", "ESPD Approach", "UDR Approach", "MDR Approach"), 
       col=c("black", "red", "blue", "green", "purple"), lty=1, lwd=2);




## Probabilistic sensitivity analysis: ESD Modeling Approach ----

# The ESD modeling was chosen to illustrate the application of the approaches in a probabilistic
# sensitivity analysis because of the waiting time that would be quite substantial if all approaches
# were to be applied. Of course, it is straightforward to implement another approach.

## Data analysis

# Number of (probabilistic sensitivity analysis) runs to be performed
n_bootstraps <- 100;

# Set seed for reproducibility
set.seed(34512);

# Perform data analysis and put the estimated parameter values in a list for each run
ESD_in_psa <- lapply(1:n_bootstraps, function(run) {
  
  # Draw a bootstrap sample from the original data
  bootstrap_sample <- IPD_censored[sample(nrow(IPD_censored), size=nrow(IPD_censored), replace=T), , drop=F];
  
  # Analyze the bootstrap sample
  ESD_analysis(time=bootstrap_sample$Time, event=bootstrap_sample$Event, censored="Censored");
  
}) # sapply


## Simulation

# Set the number of patients to be simulated
n_sim <- 1000;

# Perform the simulation for all (probabilistic sensitivity analysis) runs
ESD_out_psa <- t(sapply(ESD_in_psa, function(parameters) {
  
  # Perform the simulation
  ESD_out <- ESD_simulation(parameters=parameters, n_sim=n_sim);
  
  # Return the outcome of interest
  c(PFS.mean=mean(ESD_out$Time), Progression.prob=mean(ESD_out$Event=="Progression"));
  
})); # sapply

# Summarize and present the result of interest
summary(ESD_out_psa);
