# Function to analyze censored time-to-event data according to the
# ESD modeling approach, requiring the following input:
# time          vector of time-to-event observations
# event         vector of events corresponding to time
# censored      indicator of which event corresponds to censoring
# optim_maxit   maximum number of iterations for the 'optim' function
ESD_analysis <- function(time, event, censored, optim_maxit=1000) {
  
  # Make sure the event vector is a character vector
  event <- as.character(event);
  
  # Extract the unique events in the sample, but exclude the censoring
  # indicator as an event
  events <- sort(unique(event));
  events <- events[!events==censored];
  
  # Create a binary matrix with the censoring indicator
  # 1   experienced the event
  # 0   did not experience the event
  event_mat <- sapply(events, function(e) ifelse(event==e, 1, 0));
  
  # For each event separately, estimate the parameters
  optim_out <- sapply(events, function(e) {
    
    # Start values for the optimization
    start <- c(1, 1/mean(time[event==e]));
    
    # Optmization
    optim <- optim(par=start, fn=cs_loglik, t=time, e=event_mat[,e], control=list(maxit=optim_maxit));
    
    # Return the parameter estimates and convergence indicator
    c(shape=optim$par[1], rate=optim$par[2], convergence=optim$convergence)
    
  });
  
  # Extract all convergence indicators and parameter estimates
  convergence <- sum(optim_out["convergence",]);
  shape <- optim_out["shape",];
  rate <- optim_out["rate",];
  scale <- 1/rate;
  
  # Return the outcomes
  return(c(Shape=shape, Scale=scale, Rate=rate, Covergence=convergence));
  
}