# Function to analyze censored time-to-event data according to the
# ESPD modeling approach, requiring the following input:
# time          vector of time-to-event observations
# event         vector of events corresponding to time
# censored      indicator of which event corresponds to censoring
# max_time      time used to approximate the limit of the cumulative incidence function
# optim_maxit   maximum number of iterations for the 'optim' function
ESPD_analysis <- function(time, event, censored, max_time=100000, optim_maxit=1000) {
  
  # Make sure the event vector is a character vector
  event <- as.character(event);
  
  # Extract the unique events in the sample, but exclude the censoring
  # indicator as an event
  events <- sort(unique(event));
  events <- events[!events==censored];
  n_events <- length(events);
  
  # Create a binary matrix with the censoring indicator
  # 1   experienced the event
  # 0   did not experience the event
  event_mat <- sapply(events, function(e) ifelse(event==e, 1, 0));
  
  # First the parameters will be estimated according to the 
  # cause-specific hazards model to estimate the event-specific
  # probabilities for the ESPD approach
  optim_cs <- sapply(events, function(e) {
    
    # Start values for the optimization
    start <- c(1, 1/mean(time[event==e]));
    
    # Optmization
    optim <- optim(par=start, fn=cs_loglik, t=time, e=event_mat[,e], control=list(maxit=optim_maxit));
    
    # Return the parameter estimates and convergence indicator
    c(shape=optim$par[1], rate=optim$par[2], convergence=optim$convergence)
    
  });
  
  # Extract all convergence indicators and parameter estimates
  shape_cs <- optim_cs["shape",];
  rate_cs <- optim_cs["rate",];

  # Event probabilities (according to custom ESD)
  probs_raw <- sapply(events, function(e) cs_cif(t=max_time, k=shape_cs, p=rate_cs, i=which(events==e)));
  
  # Make sure the probabilities add up to one
  probs <- probs_raw / sum(probs_raw);
  
  # Start values for the optimization to determine the distribution parameters 
  # according to the subdistribution hazard
  start_sd <- c(rep(1, n_events), 1/sapply(events, function(i) mean(time[event==i]) ) );
  
  # Optimization
  optim_sd <- optim(par=start_sd, fn=sd_loglik, t=time, e=event_mat, control=list(maxit=optim_maxit));
  
  # Extract all convergence indicators and parameter estimates
  convergence <- optim_sd$convergence;
  shape <- optim_sd$par[c(1:n_events)];
  rate <- optim_sd$par[-c(1:n_events)];
  scale <- 1/rate;
  
  # Set appropriate names
  names(shape) = str_c("Shape.", events);
  names(scale) = str_c("Scale.", events);
  names(rate) = str_c("Rate.", events);
  names(probs) = str_c("Prob.", events);
  
  # Return the outcomes
  return(c(shape, scale, rate, probs, Covergence=convergence));
  
}