# Function to analyze censored time-to-event data according to the
# MDR modeling approach. Since the parameters estimated from the 
# ESPD modeling approach were used to define the mixture distribution
# for the MDR approach, the 'ESPD_analysis' function is called, 
# which requires the following input:
# time              vector of time-to-event observations
# event             vector of events corresponding to time
# censored          indicator of which event corresponds to censoring
# max_time          time used to approximate the limit of the cumulative incidence function
# optim_maxit       maximum number of iterations for the 'optim' function
# multinom_maxit    maximum number of iterations for the 'multinom' function
MDR_analysis <- function(time, event, censored, max_time=100000, optim_maxit=1000, multinom_maxit=1000) {
  
  # Make sure the event vector is a character vector
  event <- as.character(event);
  
  # Extract the unique events in the sample, but exclude the censoring
  # indicator as an event
  events <- sort(unique(event));
  events <- events[!events==censored];

  # Prepare data for fitting multinomial regression model using the 'multinom'
  # function of the 'nnet' package, requiring a matrix with a column for each
  # event and an 1 to indicate whether an event occured for an observation (row)
  # , or that the observation was censored (all columns 1).
  Y <- sapply(events, function(e) ifelse(event %in% c(e, censored), 1, 0));
  X <- time;
  
  # Fit multinomial logistic regression model
  mlrm <- multinom(formula=Y~X, trace=0, maxit=multinom_maxit);
  
  # Call the 'ESPD_analysis' function to estimate the parameters for the mixture distribution
  ESPD_in <- ESPD_analysis(time=time, event=event, censored=censored, max_time=max_time, optim_maxit=optim_maxit);
  
  # Return the outcomes
  return(list(mixture=ESPD_in, regmod=mlrm));
  
}