# Function to analyze censored time-to-event data according to the
# UDR modeling approach, which requires the following input:
# time              vector of time-to-event observations
# event             vector of events corresponding to time
# censored          indicator of which event corresponds to censoring
# optim_maxit       maximum number of iterations for the 'optim' function
# multinom_maxit    maximum number of iterations for the 'multinom' function
UDR_analysis <- function(time, event, censored, optim_maxit=1000, multinom_maxit=1000) {
  
  # Make sure the event vector is a character vector
  event <- as.character(event);
  
  # Extract the unique events in the sample, but exclude the censoring
  # indicator as an event
  events <- sort(unique(event));
  events <- events[!events==censored];
  
  # Create a binary matrix with the censoring indicator
  # 1   experienced the event
  # 0   did not experience the event
  event_mat <- sapply(events, function(e) ifelse(event==e, 1, 0));
  
  # Prepare data for fitting multinomial regression model using the 'multinom'
  # function of the 'nnet' package, requiring a matrix with a column for each
  # event and an 1 to indicate whether an event occured for an observation (row)
  # , or that the observation was censored (all columns 1).
  Y <- sapply(events, function(e) ifelse(event %in% c(e, censored), 1, 0));
  X <- time;
  
  # Fit multinomial logistic regression model
  mlrm <- multinom(formula=Y~X, trace=0, maxit=multinom_maxit);
  
  # Start values for the optimization
  start <- c(1, 1/mean(time));
  
  # Optmization
  optim_out <- optim(par=start, fn=cs_loglik, t=time, e=rowSums(event_mat), control=list(maxit=optim_maxit));
  
  # Extract convergence indicator and parameter estimates
  convergence <- optim_out$convergence;
  shape <- optim_out$par[1];
  rate <- optim_out$par[2];
  scale <- 1/rate;
  
  # Return the outcomes
  return(list(distribution=c(Shape=shape, Scale=scale, Rate=rate, Convergence=convergence), regmod=mlrm));
  
}