# Function to right-censor an uncensored sample of time-to-event observations
# The following input are required
# times             vector of observed (uncensored) times
# status            vector of observed events corresponding to times
# proportion        desired proportion of censoring
# classification    classification used to identify censoring in status (default=0)
# step              time steps taken in censoring the sample (default=10)
censor <- function(times, events, proportion, classification=0, step=10) {
  
  # Check whether the provided input is correct:
  # - there should be as much times observations as there are status observations
  # - NA values are not allowed for both times and status
  if(length(times)!=length(events)) stop("Object 'times' and 'status'have different lengths!");
  if(any(is.na(times)) || any(is.na(events))) stop("NA values present in the input!");
  
  # Initialize the procedure
  t.max <- floor(max(times));
  t.cens <- t.max;
  cens.prop <- 0;
  
  # Loop until the desired proportion of censoring is achieved
  while(cens.prop < proportion){
    
    # Update the current censoring time
    t.cens <- t.cens - step;
    
    # Censor all observations that occured after the censoring time
    i.cens <- ifelse(times>t.cens, T, F);
    
    # Update the times and status of the censored obsrvations
    times <- ifelse(i.cens, t.cens, times);
    events <- ifelse(i.cens, classification, events);
    
    # Re-calculate the censoring proportion
    cens.prop <- mean(events==classification);
  }
  
  # Return a data.frame with the censored sample
  return(data.frame(Time=times, Event=events))
}
