# Function that calculates the log-likelihood for the Weibull
# distribution during the maximum likelihood estimation for the
# cause-specific hazards model based on the following input 
# parameters:
# x   vector of parameter values
#     - first all shape parameters for all events
#     - next all rate parameters for all events
# t   vector of observed time points corresponding to events e
# e   vector of observed events corresponding to time points t
cs_loglik <- function(x, t, e) {
  
  # Negative parameter values are not possible
  if(any(x<0)) return(NA);
  
  # Determine number of competing events and sample size
  n.events <- length(x)/2;
  n.obs <- length(t);
  
  # Transform the time points and events into matrices, so 
  # that matrix calculations can be performed
  tt <- matrix(data=t, nrow=n.obs, ncol=n.events, byrow=F);
  ee <- as.matrix(e);
  
  # Extract the shape parameters and rate parameters
  k <- x[c(1:n.events)];
  p <- x[-c(1:n.events)];
  
  # Transform the shape and rate parameters into matrices,
  # so that matrix calculations can be performed
  kk <- matrix(data=k, nrow=n.obs, ncol=n.events, byrow=T);
  pp <- matrix(data=p, nrow=n.obs, ncol=n.events, byrow=T);
  
  # Evaluate the hazard function for each observation
  hz <- haz(tt, kk, pp);
  
  # Obtain the survival probabilties
  sv <- apply(1 - pweibull(q=tt, shape=kk, scale=1/pp), 1, prod);
  
  # Calculate the log-likelihood
  ll <- sum( log(apply((hz * sv)^ee, 1, prod)) + log(sv^(1-apply(ee, 1, sum))) );
  
  # Return the log-likelihood (minus sign for optimization)
  return(-ll);
}
